{
Locate and List all the COM ports used on the machine
The returned Stringlist List will contain the COM value, the Friendly Name and the Hardware ID (VID and PID)
}
unit ComPortList;

interface

	uses Windows, Classes, Registry, SysUtils, StrUtils,ComCtrls, Messages, Controls, StdCtrls, ExtCtrls, Forms;
//
// Functions and procedures.
//
	function FindAvailableComPorts(pList:TStrings):Integer;
    function FindComPortNames(pList: TStringList):Integer;
    function ListComPorts(nList: TstringList): Integer;
    Function LocateComPorts(): Boolean;

Var
    AmicusPort: String;
    
implementation

Uses
    Download; //, UnitPrint;
//------------------------------------------------------------------------------------------------------------------
// Search for the serial Ports
// Returns false if failed
Function LocateComPorts(): Boolean;
Var
	ComList,Parse: TStringList;
    index: Integer;
    ComText: AnsiString;

Begin
	ComList := TStringList.Create; 							// Create a stringlist to hold the COM ports found
    Parse := TStringList.Create;                            // Create a stringlist to hold the parsed comma delimited line
Try
//
// Locate the Actual Ports and place then into the stringlist "ComList"
//
    If(ListComPorts(ComList) = 0) Then   					// List the Amount of Actual COM ports available
    Begin
        ComList.Free;
        Parse.Free;
		Result := False;                                   	// Exit False If no ports found
        exit;
    End;
//
// Try and locate the Amicus port
//
	For index := 0 to (ComList.Count - 1)Do
    	Begin
        	ComText := ComList.Strings[index]; 										// Extract a line from the list of COM ports found
            Parse.CommaText := ComText;												// Parse the line into a stringlist, seperating the comma delimited values
            If(Parse.Strings[2] = 'FTDIBUS\COMPORT&VID_0403&PID_AC75') Then 			// Find the Amicus port by examining its HardwareID (VID and PID)
            	Begin
                	AmicusPort := Parse.Strings[0];
                    //PrintForm.Print(AmicusPort);
                    Break;
                End;
        End;
//
// Make sure all the ports are assigned
//
	If(AmicusPort = '') Then
    Begin
    	ComList.Free;
        Parse.Free;
        Result := False;
        Exit;
    End;

	ComList.Free;
    Parse.Free;
    Result := True;
Except
    ComList.Free;
    Parse.Free;
    Result := False;
End;
End;
//------------------------------------------------------------------------------------------------------------------
// Locate all the actual COM ports and match them to their friendly names in the registry
// The assigned StringList will be filled upon return
// The asssigned Stringlist is filled with the COM value, the Friendly name and the Hardware ID as found in the registry
// The hardware ID contains the VID and PID values if the port is a USB to Serial Type
// The format is: -
// COMn,"Friendly Name","HardwareID"
// Note.
// The Quotes surrounding the "Friendly Name" and "HardwareID" are to allow the line to be parsed using the CommaText extention to stringlists
// Otherwise the spaces would be treated as delimiters
//
// The returned value will indicate how many COM ports have been located
function ListComPorts(nList: TstringList): Integer;
var
	ComList: TStringList;
    NameList: TStringList;
    Parse: TStringList;
    AmountOfComPorts: Integer;
    AmountOfComNames: Integer;
    ActualIndex,NameIndex: Integer;
    ComText, ActualComText, NameText: AnsiString;
    index: Integer;
begin
Try
    Parse := TStringList.Create;
	ComList := TStringList.Create;
    NameList := TStringList.Create;
    AmountOfComPorts := FindAvailableComPorts(ComList);   			// List the Amount of Actual COM ports in use
    AmountOfComNames := FindComPortNames(NameList);                 // List the Names assigned to COM ports in the registry

    //for index := 0 to ComList.Count - 1 do
    //	PrintForm.Print(ComList.Strings[index]);
    //for index := 0 to NameList.Count - 1 do
    //	PrintForm.Print(NameList.Strings[index]);

//
// Loop for the amount of actual COM ports available, and filter the ports that correspond to this application
//
    for ActualIndex := 0 to (AmountOfComPorts - 1) do
    begin
        ActualComText := ComList.Strings[ActualIndex];				// Extract an Actual COM port
        //
        // Try and match the Actual COM text to a Friendly Name
        //
        for NameIndex := 0 to (AmountOfComNames - 1) do
        begin
            ComText := NameList.Strings[NameIndex];
     		Parse.CommaText := ComText;
            if(Parse.Strings[0] = ActualComText) Then
            	Begin
                    //PrintForm.Print(ActualComText + ',"' + Parse.Strings[1] + '","' + Parse.Strings[2] + '"');
                    NameText := Parse.Strings[1];
                    If (AnsiPos('Amicus18 Serial Port', Parse.Strings[1]) > 0) Then  		// Make sure it is the Amicus port
                    	Begin
            				nList.Add(ActualComText + ',"' + Parse.Strings[1] + '","' + Parse.Strings[2] + '"');
                    		//PrintForm.Print(ActualComText + ',"' + Parse.Strings[1] + '","' + Parse.Strings[2] + '"');
                        End;
                End;
        end;
    end;
    ComList.Free;
    NameList.Free;
    Parse.Free;
    Result := nList.Count;
Except
    Result := 0;
End;
end;
//------------------------------------------------------------------------------------------------------------------
// Find out how many serial ports are assigned
// The assigned Stringlist is filled with the COM names. i.e COM1, COM2 etc...
// Returns the amount of COM ports detected
function FindAvailableComPorts(pList:TStrings):Integer;
var
   Reg: TRegistry;
   Names,Items: TStringList;
   Index: integer;
begin
    Reg := TRegistry.Create;
    try
       Reg.Access := KEY_READ;
       Reg.RootKey := HKEY_LOCAL_MACHINE;
       Reg.OpenKeyReadOnly('hardware\devicemap\serialcomm');

       Names := TStringList.Create;
       Items := TStringList.Create;
       Items.Sorted := true;
       Items.Duplicates := dupIgnore;
       try
          Reg.GetValueNames(Names);
          for index := 0 to Names.Count - 1 do
          	begin
             	Items.Add(Reg.Readstring(Names.strings[Index]));
                //PrintForm.Print(Reg.Readstring(Names.strings[Index]));	// Display the COM ports
                Application.ProcessMessages;
            end;
       except
       end;
       pList.Assign(Items);
       Names.Free;
       Items.Free;
       Reg.CloseKey;
    finally
       Reg.Free;
    end;
    Result := pList.Count;
end;
//------------------------------------------------------------------------------------------------------------------
// Find and List the Com Port Device Descriptions. i.e Friendly Names and COM assigned
// Returns the amount of Com Names Found
// The assigned Stringlist is filled with the COM value, the Friendly name and the Hardware ID as found in the registry
// The hardware ID contains the VID and PID values if teh port is a USB to Serial Type
// The format is: -
// COMn,"Friendly Name","HardwareID"
// The Quotes surrounding the "Friendly Name" and "HardwareID" are to allow the line to be parsed using the CommaText extention to stringlists
// Otherwise the spaces would be treated as delimiters
function FindComPortNames(pList: TStringList):Integer;
Var
	Reg: TRegistry;
    RootKeyNames,SubKeyNames,Sub1KeyNames,Sub2KeyNames: TStringList;
   	RootKeyIndex,SubKeyIndex,Sub1KeyIndex,Sub2KeyIndex: Integer;
    REG_KEY: String;
    ClassDesc: String;
    PortName: String;
    FriendlyName: String;
    HardWareID: String;
    AmountOfRootKeys, AmountOfSubKeys,AmountOfSub1Keys,AmountOfSub2Keys: Integer;
    DataType: Cardinal;
  	DataSize: Cardinal;
Begin
    PortName := '';
    Reg := TRegistry.Create;
try
    REG_KEY := '\SYSTEM\CurrentControlSet\Enum';
    Reg.Access := KEY_READ;
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKeyReadOnly(REG_KEY);

    RootKeyNames := TStringList.Create;
    SubKeyNames := TStringList.Create;
    Sub1KeyNames := TStringList.Create;
    Sub2KeyNames := TStringList.Create;

    try
        Reg.GetKeyNames(RootKeyNames);
        AmountOfRootKeys := RootKeyNames.Count;
        //
        // At this point, we have all the files in the ENUM folder
        // Open Each Folder and look for the CLASS 'Port'
        //
        for RootKeyIndex := 0 to AmountOfRootKeys - 1 do
        begin
        	//Form1.ASCII_RECEIVE_MEMO.Lines.Add(REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex]);
            //
            // Open the Folders within the Root Folder
            //
            Reg.OpenKeyReadOnly(REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex]);
            Reg.GetKeyNames(SubKeyNames);
            AmountOfSubKeys := SubKeyNames.Count;
            for SubKeyIndex := 0 to AmountOfSubKeys - 1 do
            begin
            	//Form1.ASCII_RECEIVE_MEMO.Lines.Add('    ' + REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex]);
                //
            	// Open the Folders within the Root Folder
            	//
                try
                Reg.OpenKeyReadOnly(REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex]);
                Reg.GetKeyNames(Sub1KeyNames);
                AmountOfSub1Keys := Sub1KeyNames.Count;
                for Sub1KeyIndex := 0 to AmountOfSub1Keys - 1 do
            	begin
                 	//Form1.ASCII_RECEIVE_MEMO.Lines.Add('        ' + REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex] + '\' + Sub1KeyNames.strings[Sub1KeyIndex]);
                    //
            		// Open the Folders within the Root Folder
                    // This should isolate the Class Type. i.e. Ports, USB, DiskDrive etc..
            		//
                    try
                	Reg.OpenKeyReadOnly(REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex] + '\' + Sub1KeyNames.strings[Sub1KeyIndex]);

                    ClassDesc := Reg.Readstring('Class');						// Read the Class Type. i.e. Ports, BlueTooth, DiskDrive etc...
                    if(ClassDesc = 'Ports') Then                                // Is the Class 'Ports'?
                    begin                                                      	// Yes. So...
                        FriendlyName := Reg.Readstring('DeviceDesc');  			// Read the Friendly Name of the Port
                        //PrintForm.Print(FriendlyName);
                        //
                        // Read the Hardware ID (if it's there) that will contain the VID and PID values
                        // The HardwareID value is a REG_MULTI_SZ type value so it requires a rather convoluted read method
                        //
                        Try
    						DataSize := 0;
    						RegQueryValueEx(
      							Reg.CurrentKey,     	// handle of key to query
      							'HardWareID',       	// address of name of value to query
                                nil,                	// reserved
                                @DataType,          	// address of buffer for value type
                                nil,                	// address of data buffer
                                @DataSize);         	// address of data buffer size
    						SetLength(HardWareID, DataSize - 1);
    						if DataSize > 1 then
                            	RegQueryValueEx(
                                	Reg.CurrentKey,     // handle of key to query
                                    'HardWareID',    	// address of name of value to query
                                    nil,                // reserved
                                    @DataType,          // address of buffer for value type
                                    PByte(HardWareID),  // address of data buffer
                                    @DataSize);         // address of data buffer size
                        Except
                        	HardWareID := '';
                        End;
                        //PrintForm.Print(HardWareID);
                        if(FriendlyName <> '') Then                             // Is the Friendly Name Empty ?
                        begin                                                   // No. So...
                			Reg.GetKeyNames(Sub2KeyNames);
                			AmountOfSub2Keys := Sub2KeyNames.Count;
                			for Sub2KeyIndex := 0 to AmountOfSub2Keys - 1 do
            				begin
                 				//Form1.ASCII_RECEIVE_MEMO.Lines.Add('            ' + REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex] + '\' + Sub1KeyNames.strings[Sub1KeyIndex] + '\' + Sub2KeyNames.strings[Sub2KeyIndex]);
                        		//
            					// Open the Folders within the Root Folder
                    			// This should isolate the Device Parameters folder holding the PortName. i.e. COM39, COM40 etc...
            					//
                    			try
                				Reg.OpenKeyReadOnly(REG_KEY + '\' + RootKeyNames.strings[RootKeyIndex] + '\' + SubKeyNames.strings[SubKeyIndex] + '\' + Sub1KeyNames.strings[Sub1KeyIndex] + '\' + Sub2KeyNames.strings[Sub2KeyIndex]);
                            	PortName := Reg.Readstring('PortName');						// Read the PortName. i.e. COM39, COM40 etc...
                                HardWareID := AnsiReplaceStr(HardWareID,#13,'');
                                HardWareID := AnsiReplaceStr(HardWareID,#12,'');
                                HardWareID := AnsiReplaceStr(HardWareID,#10,'');
                                HardWareID := Trim(HardWareID);
                            	if(PortName <> '') Then                                     // Is the PortName Empty?
                                    pList.Add(PortName + ' , "' + FriendlyName + '" , "' + HardWareID + '"');    // Add the Com ports, Friendly Names and HardWareID to the return stringlist
                                    //PrintForm.Print(PortName + ',"' + FriendlyName + '","' + HardWareID + '"');
                                	//pList.Add(PortName + ',"' + FriendlyName + '"');
                    			except
                    			end;
                			end;
                        end;
                    end;
                    except
                    end;
                end;
                except
                end;

            end;
        end;
    except
    end;
    RootKeyNames.Free;
    SubKeyNames.Free;
    Sub1KeyNames.Free;
    Sub2KeyNames.Free;
    Reg.CloseKey;
finally
	Reg.Free;
end;
    Result := pList.Count;
End;
//------------------------------------------------------------------------------------------------------------------

end.
