{
 Amicus18 bootloader
 The source code is offered without any support, or obligations
 Use it and alter it as you wish, but if any changes are made that improve it's operation
 please offer it to the rest of the Amicus community.

 However, always include the source code and document the changes made
}
unit Download;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Menus, Gauges, Grids, cMMServer;

type
  TDownloadForm = class(TForm)
    DelayTimer: TTimer;
    Panel1: TPanel;
    WriteButton: TBitBtn;
    CloseButton: TBitBtn;
    Panel2: TPanel;
    Panel3: TPanel;
    DownloadGauge: TGauge;
    Edit2: TLabel;
    Panel4: TPanel;
    StartUpTimer: TTimer;

    Function StartDownloading (pHexfileName: string; pSystem: integer; pPort: integer): Boolean;
    Function BuildCodeBuffer(pRecordType: String ; pLineBuffer : String ; pAddress : Dword) : Boolean;
    Function BuildDataBuffer(pRecordType: String ; pLineBuffer : String ; pAddress : Dword) : Boolean;
    Function OpenCom(pActiveComPort: String) : Boolean;
    Procedure CloseCom();
    Function SendDataToMicrocontroller(pRecordType: String; pLineBuffer : string) : Boolean;
    Procedure FormCloseQuery(Sender: TObject; Var CanClose: Boolean);
    Procedure DelayTimerTimer(Sender: TObject);
    Procedure CloseButtonClick(Sender: TObject);
    Procedure WriteButtonClick(Sender: TObject);
    Procedure FormActivate(Sender: TObject);
    Function WriteByte(pCharacter: Byte) : Boolean;
	Function ReadByte() : Byte;
    Function WaitForACK() : Boolean;
    Procedure DisplayMessage(pMessageBuffer: String);
    Procedure DelayMs(pDelayTime: Integer);
    Procedure DoError(pErrorText: String);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure StartUpTimerTimer(Sender: TObject);

  private
    { Private declarations }
    Global_OKToDownload, Global_DownloadInProgress: Boolean;
    hCom: THandle;
    Directory: string;
    Global_HexFile: string;
    Global_CloseButtonPressed:  Boolean;
    Global_ComPort: String;
    Global_DownloadStatus:  Boolean;					// Is set true If a succesful Write took place
  	DevName: array [0..20] of char;                     // device name
  public
    { Public declarations }
  End;

Var
  DownloadForm: TDownloadForm;
  Global_ComPortOpen: Boolean;                          // Is true if the com port is open
  Global_HexfileOpen: Boolean;                          // Is true if the HEX file is open
  Global_WriteCount: Byte;							    // Counts the bytes or words written/read to/from the microcontroller
  Global_UpperAddress: Word;							// Used in INHX32 format hex file for high word Of address
  Global_ErrorReported: Boolean;

  Global_DeviceSize: Word;
  Global_DeviceType: Byte;                          	// Stores the device's unique identifier type

  Global_ByteCount: Dword;

  Global_BufferCount: Byte;
  TBLPTRU: Byte;
  TBLPTRH: Byte;
  TBLPTRL: Byte;
  Global_Flags: Byte;
  Global_CrcChecksum: Word;

  Global_LoaderStartAddress: Dword;			            // Address of bootloader in Code memory.
  Global_CodeBuffer: Array [0..131071] Of Word; 	    // Holds all the bytes extracted from the hex file for Code memory
  Global_DataBuffer: Array [0..1023] Of Word; 	        // Holds all the bytes extracted from the hex file for Data memory

const
//
// HEX file record types
//
    DATA_REC = '00';
    EOF_REC = '01';
    SEGMENT_REC = '02';
    LINEAR_REC = '04';
//
// microcontroller details, based upon the microcontroller type used in the Amicus18
//
    cBootloaderSize = 256;
    cAmountOfCodeMemory = 32768;
    cAmountOfEepromMemory = 255;

  ApplicationName = 'Amicus18 Loader';

implementation

Uses
    ComPortList;

{$R *.DFM}
{----------------------------------------------------------------------------------------}
// Form Activate
Procedure TDownloadForm.FormActivate(Sender: TObject);
Var
  	S: string;
Begin
    MMServer.WritePacket('[loader]on');  	// Transmit a server message
    Global_ComPortOpen := False;                                              // Default to the com port not open
    Global_HexfileOpen := False;                                              // Default to the HEX file not open
  	Global_DownloadInProgress := False;

  	Directory := ExtractFileDir(Application.ExeName);
  	If (Length(Directory) > 3) Then
  		Begin
    		Directory := Directory + '\';
  		End;
    MMServer.WritePacket('[loader]on');  	// Transmit a server message
  	S := ParamStr(1);
  	If S <> '' Then Global_HexFile := S;

  	If S = '' Then
   		DownloadForm.Caption := 'Amicus18 Loader'
  	Else
  		DownloadForm.Caption := AnsiLowerCase(ExtractFileName(S));

  	Application.ProcessMessages;

  	If (FileExists(Global_HexFile)) Then
    	Global_OKToDownload := True
  	else
    	Global_OKToDownload := False;
    MMServer.WritePacket('[loader]on');  	// Transmit a server message
    Application.ProcessMessages;
    DelayMs(100);
//
// Locate the Com Port that the bootloader is attached to
//
   If(LocateComPorts() = false) Then
   	Begin
        DoError('Cannot find the Amicus18'); 
    	exit;
    End;
   Global_ComPort := AmicusPort;

   StartUpTimer.Enabled := True;

   CloseCom();
   DelayMs (4);
   Close();
End;
{----------------------------------------------------------------------------------------}
// Startup Timer to initiate the downloader
procedure TDownloadForm.StartUpTimerTimer(Sender: TObject);
begin
	StartUpTimer.Enabled := False;
    WriteButtonClick(Sender);               // Start the download
end;

{----------------------------------------------------------------------------------------}
// Write button pressed
Procedure TDownloadForm.WriteButtonClick(Sender: TObject);
Var
	ClearCounter: Dword;
Begin

    Global_ErrorReported := False;						// Indicate that no error has been displayed
    // Clear the Code storage array
    // High equal to $FF means no code
    For ClearCounter := 0 to 131071 Do
    	Begin
        	Global_CodeBuffer[ClearCounter] := $FFFF;
        End;
    // Clear the Data storage array
    // High equal to $FF means no code
    For ClearCounter := 0 to (cAmountOfEepromMemory - 1) Do
    	Begin
        	Global_DataBuffer[ClearCounter] := $FFFF;
        End;

	If Global_OKToDownload Then
    	Begin
        	DisplayMessage ('');
        	StartDownloading (Global_HexFile, 1, 1);
    	End
    Else
    	Begin
			DoError ('Cannot Read HEX file.');
    		Global_DownloadStatus := False;				// Indicate a BAD Write
    	End;

    If(Global_DownloadStatus = True)Then
    	Begin
            CloseCom();                             // CLose the com file if it's still open
            DelayMs(4);
        	Close();
    	End;
End;
{----------------------------------------------------------------------------------------}
// Close Form Query
Procedure TDownloadForm.FormCloseQuery(Sender: TObject; Var CanClose: Boolean);
Begin
    CloseCom();
    MMServer.WritePacket('[loader]off');
End;
{----------------------------------------------------------------------------------------}
// Returns true if a succesful download took place
Function TDownloadForm.StartDownloading (pHexfileName: string; pSystem: integer; pPort: integer): Boolean;
Var
  	HexFile: Textfile;
  	HexData: string;
  	SuccesfulWrite: Boolean;
  	LineNumber : Integer;
    pRecordType: String;
    HexFileOK: Boolean;

    Address : Integer;
    EepromStartAddress : Integer;
  	ID_START : Integer;

    ScanCount: Dword;
    BufferCount: Byte;
    DataAvailableWrite: Boolean;
Begin
  	Global_DownloadStatus := True;    							                // Default to a good Write

    //Memo1.Clear();

    If (OpenCom(PChar(AmicusPort)) = False) Then 				                // Open COM port. Exit if a fault occured
        Begin
            Result := False;
    		Exit;
        End;

    DownloadGauge.ForeColor := clLIME;
    DownloadGauge.Progress := 0;
    Global_DownloadInProgress := True;
    Global_HexfileOpen := False;                                                // Default to the HEX file being closed
    AssignFile(HexFile, pHexfileName);
    Global_HexfileOpen := True;                                                 // Set the flag for HEX file open
//
// Read through the HEX file and examine it for any anomalies
// Also examine the first line Of HEX to see if it's a 32 bit format
// If it is, then extract the RecordType
// Also find the length (in lines) Of the HEX file for the download gauge's maximum value.
//
    DisplayMessage ('Scanning HEX File');
    LineNumber := 0; 															// Reset the line count
    HexFileOK := True;														    // Default to a good HEX file format
    Reset(HexFile); 															// Start at the beginning Of the HEX file
    While (NOT EOF(HexFile)) Do 												// Loop until the End Of the HEX file is reached
    	Begin
        	Readln(HexFile, HexData);
            If (Length(HexData) <> 0) Then 									    // Continue if the line is not empty
    			Begin
                    If (HexData[1] = ':') Then 								    // Does the line of HEX start with a colon ?
            			Begin
                            If(LineNumber = 0) Then	                            // Do the next block of code only on the first line
                            	Begin
                                    pRecordType := HexData[8] + HexData[9]; 	// Find the Record Type of the hex file
                                    If(pRecordType <> LINEAR_REC) Then  		// is HEX file INHX32 format
                    					Begin
                        					DoError ('Incorrect HEX file format'); // Record Type not recognised so issue an error
                                            HexFileOK := False;				    // Signal an error occured
            								Break;        						// And exit the while loop prematurely
                        				End;
                                End;
                            End
            		Else
            			Begin
                			DoError ('Invalid HEX file structure'); 			// No colon found so issue an error
                            HexFileOK := False;		   						    // Signal an error occured
            				Break;               								// And exit the while loop prematurely
                		End;
                    Inc (LineNumber); 											// Build the number Of lines in the HEX file
        		End
            Else
            	Begin
                	DoError ('Invalid HEX file length'); 						// Issue an error if the line was empty
                    HexFileOK := False;										    // Signal an error occured
            		Break;                                  					// And exit the while loop prematurely
                End;
        End;

    If (HexFileOK = False) Then 						                        // Exit false if an error was found while reading the HEX file
    	Begin
        	Result := False;
            If (Global_HexfileOpen = True) Then                                 // Is the HEX file still open ?
                Begin                                                           // Yes. So...
                    CloseFile(HexFile);                                         // Close the HEX file
                    Global_HexfileOpen := False; 					            // Signal that it's closed
                End;
    		Exit;
        End;

//
// Search for the bootloader
//
	WriteButton.Enabled := False;							                    // Disable the Write button while downloading
    Global_CloseButtonPressed := False;                                         // Reset the close Button's trigger variable
//
// Strobe DTR LOW-HIGH-LOW
//
    EscapeCommFunction(hCom, CLRDTR);
    DelayMs (20);
    EscapeCommFunction(hCom, SETDTR);
    DelayMs (20);
    EscapeCommFunction(hCom, CLRDTR);

    DisplayMessage ('Press Reset on the Amicus18');
//
// Wait for an acknowledge from the firmware and it's type
// microcontroller type is returned in variable Global_DeviceType
//
    If((WaitForACK()) = False) Then
    	 Begin
            Result := False; 					                                // Signal an error occured
            If (Global_HexfileOpen = True) Then                                 // Is the HEX file still open ?
                Begin                                                           // Yes. So...
                    CloseFile(HexFile);                                         // Close the HEX file
                    Global_HexfileOpen := False; 					            // Signal that it's closed
                End;
    		Exit;											                    // And exit prematurely
        End;
//
// Check for the Close button being pressed
//
    If(Global_CloseButtonPressed = True) Then
        Begin
            DoError ('Download Cancelled');
            WriteButton.Enabled := True;
            Result := False; 										            // Signal an error occured
            If (Global_HexfileOpen = True) Then                   	            // Is the HEX file still open ?
                Begin                                       		            // Yes. So...
                    CloseFile(HexFile);                    			            // Close the HEX file
                    Global_HexfileOpen := False; 					            // Signal that it's closed
                End;
    		Exit;													            // And exit prematurely
        End;
//
// Start loading the HEX file and build up the data in two arrays. One for Code and one for Data memory
//
    DataAvailableWrite := False;
    LineNumber := 0; 															    // Reset the line count
    SuccesfulWrite := True;													        // Default to no errors
    DownloadGauge.ForeColor := clLime;											    // Change the colour Of the download guage to lime
    Application.ProcessMessages();
    Global_UpperAddress := 0;													    // Default to an Global_UpperAddress Of 0;
    Reset(HexFile); 															    // Point back to beginning Of the HEX file
    DisplayMessage ('Writing Program to Amicus18');
    While(not EOF(HexFile)) and (Global_CloseButtonPressed = False) and (SuccesfulWrite = True) Do
    	Begin
            If (LineNumber = 0) Then Readln(HexFile, HexData);                     // Ignore the first line Of HEX

            Readln(HexFile, HexData);
            pRecordType := HexData[8] + HexData[9]; 								// Find the record type Of the HEX line
            If (pRecordType = LINEAR_REC) Then 										// Have we found a linear record line?
                Begin
            		Global_UpperAddress := StrToIntDef('$' + HexData[12] + HexData[13],0); 	// Extract the upper address
                	Readln(HexFile, HexData);                 					    // *** and read the next line ***
                End;

            If (pRecordType = EOF_REC) Then 										// Have we reached the End Of Record value 01 ?
            	Break                                								// Yes. So exit the while loop
            Else                              										// Otherwise send the line of HEX data to the microcontroller

    			Address := StrToIntDef('$' + HexData[4] + HexData[5] + HexData[6] + HexData[7],2); // Addresses in words.
				Address := (Global_UpperAddress * 65536) + Address;          	    // Add the upper address byte to the 24-bit address
    			ID_START := $200000;
    			EepromStartAddress := $F00000;

                If ((Address <= ID_START) OR (Address >= EepromStartAddress)) Then
                Begin
                	If (Address < EepromStartAddress) Then
    					Begin
                         	//
                            // Build the Code memory array
                            //
                			If(BuildCodeBuffer (pRecordType , HexData , Address) = False) Then
                				Begin
                    				SuccesfulWrite := False;								// Signal an error occured while building the byte information
                        			Break;  												// And exit the while loop prematurely;
            					End;
                    	End
                	Else
                		Begin
                        	//
                            // Build the Data memory array
                            //
                            DataAvailableWrite := True;										// Signal that there is eeprom (data) memory to write 
                        	If(BuildDataBuffer (pRecordType , HexData , Address) = False) Then
                				Begin
                    				SuccesfulWrite := False;								// Signal an error occured while building the byte information
                        			Break;  												// And exit the while loop prematurely;
            					End;
                    	End;
                	End;
        		Inc(LineNumber);
        		End;

        If (SuccesfulWrite = True) Then
        Begin
        			//
        			// Take the first 8 bytes and place them at vector location
        			//
            		Global_LoaderStartAddress := cAmountOfCodeMemory - cBootloaderSize;
        			For Global_ByteCount := Global_LoaderStartAddress To Global_LoaderStartAddress + 63 Do 			// Initialise address
        				Begin
            				Global_CodeBuffer[Global_ByteCount] := $FF;
            				If (Global_ByteCount = Global_LoaderStartAddress + 56) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[0];
                			If (Global_ByteCount = Global_LoaderStartAddress + 57) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[1];
                			If (Global_ByteCount = Global_LoaderStartAddress + 58) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[2];
                			If (Global_ByteCount = Global_LoaderStartAddress + 59) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[3];
                			If (Global_ByteCount = Global_LoaderStartAddress + 60) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[4];
                			If (Global_ByteCount = Global_LoaderStartAddress + 61) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[5];
                			If (Global_ByteCount = Global_LoaderStartAddress + 62) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[6];
                			If (Global_ByteCount = Global_LoaderStartAddress + 63) Then Global_CodeBuffer[Global_ByteCount] := Global_CodeBuffer[7];
            			End;
                    //
                    // Bytes to place at address 00 to 07 of microcontroller. Creates a Goto Bootloader mnemonic
                    //
        			Global_CodeBuffer[0] := $A0;
        			Global_CodeBuffer[1] := $EF;
        			Global_CodeBuffer[2] := $3F;
        			Global_CodeBuffer[3] := $F0;
        			Global_CodeBuffer[4] := $FF;
        			Global_CodeBuffer[5] := $FF;
        			Global_CodeBuffer[6] := $FF;
        			Global_CodeBuffer[7] := $FF;

            		DownloadGauge.MaxValue := Global_LoaderStartAddress + 64;
        			Global_ByteCount := 0;
        			SuccesfulWrite := True; 													// Default to a succesful write
        			While ((Global_ByteCount <= Global_LoaderStartAddress) and (Global_CloseButtonPressed = False))Do 	// Create a loop for all Code memory
        			Begin
            			Global_CrcChecksum := 0; 												// Clear CRC checksum before we start
            			BufferCount := 0;                                                     	// Clear Global_ByteCount checksum before we start
						ScanCount := Global_ByteCount;
            			If ((Global_CodeBuffer[ScanCount] SHR 8) and 255 <> $FF) Then 			// Look for a valid byte to send
            			Begin
                    		WriteByte((Global_ByteCount SHR 16) and 255);						// Send the upper address to the microcontroller firmware
                    		Global_CrcChecksum := Global_CrcChecksum + (Global_ByteCount SHR 16) and 255;

                    		WriteByte((Global_ByteCount SHR 8) and 255);						// Send the high address to the microcontroller firmware
                    		Global_CrcChecksum := Global_CrcChecksum + (Global_ByteCount SHR 8) and 255;

                   	 		WriteByte(Global_ByteCount and 255);								// Send the low address to the microcontroller firmware
                    		Global_CrcChecksum := Global_CrcChecksum + Global_ByteCount and 255;

                    		WriteByte(64);														// Send the amount of bytes to the microcontroller firmware

                    		Global_CrcChecksum := Global_CrcChecksum + 64;

                 			While (((Global_CodeBuffer[ScanCount] SHR 8) and 255 <> $FF) and (BufferCount < 64))Do
        					Begin
           						WriteByte(Global_CodeBuffer[ScanCount] and 255);				// Send the byte to the microcontroller firmware
                        		Global_CrcChecksum := Global_CrcChecksum + (Global_CodeBuffer[ScanCount] and 255);
           						Inc (ScanCount);
           						Inc(BufferCount);
                				Inc (Global_ByteCount);
                        		DownloadGauge.Progress := Global_ByteCount;
           	 					Application.ProcessMessages();
                			End;

                    		While (BufferCount < 64) Do
        					Begin
             					WriteByte(Global_CodeBuffer[ScanCount] and 255);				// Send the byte to the microcontroller firmware
                        		Global_CrcChecksum := Global_CrcChecksum + (Global_CodeBuffer[ScanCount] and 255);
                				Inc (ScanCount);
            					Inc(BufferCount);
                				Inc (Global_ByteCount);
                        		DownloadGauge.Progress := Global_ByteCount;
           	 					Application.ProcessMessages();
            				End;

                    		WriteByte(256 - (Global_CrcChecksum and 255));						// Send the CRC checksum to the microcontroller firmware
            				Global_CrcChecksum := ReadByte();
                    		If (Global_CrcChecksum <> $4B) Then
                    			Begin
                         			SuccesfulWrite := False;
                                	DoError('Download failed at Code address $' + IntToHex(Global_ByteCount,4));
                            		Break;
                        		End;
                    	End
            		Else
            			Begin
            				Inc (Global_ByteCount);
                    		DownloadGauge.Progress := Global_ByteCount;
                    		Application.ProcessMessages();
                		End;
                	End;
        		//
        		// Now Do Data memory
        		//
            	If (SuccesfulWrite = True) Then
        		Begin
                    //
                    // Check if there is any eeprom data to write
                    //
                    If(DataAvailableWrite = True) Then
                    	Begin
                    		DisplayMessage ('Programming Eeprom Memory');
            				DownloadGauge.MaxValue := cAmountOfEepromMemory;
        					Global_ByteCount := 0;
        					SuccesfulWrite := True; 											// Default to a succesfull write
        					While ((Global_ByteCount <= cAmountOfEepromMemory) and (Global_CloseButtonPressed = False))Do 	// Create a loop for all Data memory
        						Begin
            						Global_CrcChecksum := 0; 												// Clear CRC checksum before we start
                    				WriteByte(64);															// Send the flag to the microcontroller firmware
                    				Global_CrcChecksum := Global_CrcChecksum + 64;

                        			TBLPTRH := Global_ByteCount;
                    				WriteByte(TBLPTRH);														// Send the address to the microcontroller firmware
                                    Global_CrcChecksum := Global_CrcChecksum + TBLPTRH;
                                    
                        			WriteByte(Global_DataBuffer[Global_ByteCount] and 255);					// Send the byte to the microcontroller firmware
                        			Global_CrcChecksum := Global_CrcChecksum + (Global_DataBuffer[Global_ByteCount] and 255);

                        			Inc(Global_ByteCount);
                        			DownloadGauge.Progress := Global_ByteCount;
                        			Application.ProcessMessages();

                    				WriteByte(0);															// Send the amount of bytes to the microcontroller firmware
                    				Global_CrcChecksum := Global_CrcChecksum + 0;
                        			DelayMs (1);
                    				WriteByte(256 - (Global_CrcChecksum and 255));							// Send the CRC checksum to the microcontroller firmware
                        			Global_CrcChecksum := ReadByte();
                        			If (Global_CrcChecksum <> $4B) Then
                    					Begin
                         					SuccesfulWrite := False;
                                			DoError('Download failed at Eeprom address $' + IntToHex(Global_ByteCount,4));
                            				Break;
                        				End;
                				End;
                		End;
        		End;
        End;
//
// HEX file read and downloaded, so check if it was succesful, and start finalising the downloading process
//
    If (SuccesfulWrite = True) Then
    	Begin
            DownloadGauge.ForeColor := clLIME;
            DownloadGauge.Progress := Global_LoaderStartAddress + 64;

            DisplayMessage ('Program Written Successfully');
            WriteButton.Enabled := True;
            Global_DownloadStatus := True;								 // Indicate a GOOD Write
        End;

    If (Global_CloseButtonPressed = True)Then
    	Begin
    		DoError ('Download Cancelled');
            Result := False;
            If (Global_HexfileOpen = True) Then                         // Close the HEX file if it's still open
                Begin
                    Global_HexfileOpen := False;
                    CloseFile(HexFile);
                End;
            Exit;
        End
    Else
        Begin
     		If(SuccesfulWrite = False) Then
                Begin
                    //If (Global_DownloadStatus = True) Then
                    DoError ('Download Failed');
            		Result := False;
                    If (Global_HexfileOpen = True) Then
                        Begin
                            Global_HexfileOpen := False;
                            CloseFile(HexFile);
                        End;
                    Exit;
                End;

        End;

    If (Global_HexfileOpen = True) Then                   // Close the HEX file if it's still open
        Begin
            Global_HexfileOpen := False;
            CloseFile(HexFile);
        End;
    Result := True;
    CloseCom ();
End;
//----------------------------------------------------------------------------------------
// Construct the Code memory buffer to send to the microcontroller firmware
// Buffer structure
// TBLPTRU value. Includes 2 flags bit 7 set for CONFIG memory, bit 6 set for Data memory
// TBLPTRH
// TBLPHRL
// 64 bytes of data
// CRC checksum of all above values added togrther

Function TDownloadForm.BuildCodeBuffer(pRecordType: String ; pLineBuffer : String; pAddress : Dword) : Boolean;
Var
	AmountOfWords: Byte;
    N: Byte;
    Pointer: Byte;
Begin

    AmountOfWords := (StrToIntDef('$' + pLineBuffer[2] + pLineBuffer[3],1)); 		// Number of words on the line

    For N := 0 To AmountOfWords - 1 Do
    	Begin
        	Pointer := N * 2;
            Global_CodeBuffer[pAddress] := StrToIntDef('$' +  pLineBuffer [10 + Pointer] + pLineBuffer[11 + Pointer],0);
            Inc(pAddress);
        End;

    Result := True;
End;
//----------------------------------------------------------------------------------------
// Construct the Data memory buffer to send to the microcontroller firmware
Function TDownloadForm.BuildDataBuffer(pRecordType: String ; pLineBuffer : String ; pAddress : Dword) : Boolean;
Var
	AmountOfBytes: Byte;
    N: Byte;
    Pointer: Byte;
Begin
    AmountOfBytes := (StrToIntDef('$' + pLineBuffer[2] + pLineBuffer[3],1)); 	// Number of bytes on the line

    pAddress := pAddress and $FFFF;				// Strip off the upper byte of the address
    N := 0;
    Repeat
    	Pointer := N * 2;
        Global_DataBuffer[pAddress] := StrToIntDef('$' +  pLineBuffer [10 + Pointer] + pLineBuffer[11 + Pointer],0);
        Inc(pAddress);
    	Inc(N);
    Until (N > AmountOfBytes - 1);
    Result := True;
End;
//---------------------------------------------------------------------------------------
// Write words to the bootloader firmware
Function TDownloadForm.SendDataToMicrocontroller(pRecordType: String ; pLineBuffer : String) : Boolean;
Begin
    Result := True;
End;
//---------------------------------------------------------------------------------------
// Open the COM port
// And issue errors if not able to open or change settings
// Function returns true is everything OK, else returns false
Function TDownloadForm.OpenCom(pActiveComPort: String) : Boolean;
Var
  	fSuccess : Boolean;
  	dcb : TDCB;
    s:string;
    LDevName: array [0..20] of char;
  	TimeOuts : COMMTIMEOUTS;
Begin
    // make a com name of the com
   strPCopy(@devName, pActiveComPort);

   // make a device name, as we need a device name to open com9 and above
   s := '\\.\'+pActiveComPort;
   strPCopy(@LdevName, s);

    Global_ComPortOpen := False;                // Default to the com port closed
	Application.ProcessMessages;
  	hCom := CreateFile
    	(
        	LDevName,        					// Open the COM port
  			GENERIC_READ or GENERIC_WRITE,
  			0,                                  // Exclusive access
  			NIL,                                // No security attrs
  			OPEN_EXISTING,
  			FILE_ATTRIBUTE_NORMAL,
  			0
  		);

  	If(hCom = INVALID_HANDLE_VALUE) Then
    	Begin
    		DoError ('Problem Opening Port ' + String(pActiveComPort));
    		Result := False;					// Return an error status
    		exit;
  		End;

  	fSuccess := GetCommState(hCom, dcb);
  	If(NOT fSuccess ) Then
    	Begin
    		DoError ('Problem Reading ' + String(pActiveComPort) +  ' Parameters.');
    		Result := False;					// Return an error status
    		exit;
  		End;

  	dcb.BaudRate := 115200;                     //Set up UART parameters
  	dcb.ByteSize := 8;
  	dcb.Parity := NOPARITY;
  	dcb.StopBits := ONESTOPBIT;
  	dcb.Flags := $00000001;
  	fSuccess := SetCommState(hCom, dcb);    	//Ensure changes were carried out succesfully
  	If(NOT fSuccess) Then
    	Begin
    		DoError ('Problem Writing ' + String(pActiveComPort) +  ' Parameters.');
    		Result := False;  					// Return an error status
    		exit;
  		End;

    GetCommTimeouts(hCom,TimeOuts);            //Set timeouts for UART
  	TimeOuts.ReadIntervalTimeout := 100;
  	TimeOuts.ReadTotalTimeoutMultiplier := 100;
  	TimeOuts.ReadTotalTimeoutConstant := 100;
  	TimeOuts.WriteTotalTimeoutMultiplier := 100;
  	TimeOuts.WriteTotalTimeoutConstant := 100;
  	fSuccess := SetCommTimeouts(hCom,TimeOuts);

  	If(NOT fSuccess) Then
    	Begin
    		DoError ('Problem Setting ' + String(pActiveComPort) +  ' Timeouts.');
    		Result := False;   				// Return an error status
    		exit;
  		End;
    //clear the serial RX/TX buffers
  	fSuccess := PurgeComm(hCom,PURGE_TXABORT + PURGE_RXABORT + PURGE_TXCLEAR + PURGE_RXCLEAR);
  	If(NOT fSuccess) Then
    	Begin
    		DoError ('Problem Clearing ' + String(pActiveComPort) +  ' Buffers.');
    		Result := False;  					// Return an error status
    		exit;
  		End;
    Global_ComPortOpen := True;                       // Set the flag that tells the rest of thge compiler that the com port is open
  	Result := True; 							// Indicate COM opened OK
End;
{----------------------------------------------------------------------------------------}
// Close port
Procedure TDownloadForm.CloseCom();
Begin
Try
    If (Global_ComPortOpen = True) Then CloseHandle(hCom);
    Global_ComPortOpen := False;
Except
	Global_ComPortOpen := False;
End;
End;
{----------------------------------------------------------------------------------------}
Procedure TDownloadForm.DelayTimerTimer(Sender: TObject);
Begin
  	DelayTimer.Enabled := False;
End;
//---------------------------------------------------------------------------------------
// Wait for the ACK from the Firmware
// Send the byte $C1 repeatedly, until a valid byte is received
// The character K is then received
// Returns false if an error occured
// Otherwise returns true
Function TDownloadForm.WaitForACK() : Boolean;
Var
	ReturnResult: Boolean;
Begin
    ReturnResult := False; 										// Default to no ack found
    PurgeComm(hCom,PURGE_TXCLEAR + PURGE_RXCLEAR);
	While (ReturnResult = False) and (Global_CloseButtonPressed = False) Do
    	Begin
            WriteByte($C1);  									// Send the IDENT byte $C1'
            Global_DeviceType := ReadByte();					// Receive a byte
            If(Global_DeviceType <> $00) Then   				// Have we received a valid byte  ?
               	Begin
                	ReturnResult := True;  						// Yes.. So signal a good reception
                    Break;                                      // And break out Of the loop
                End;
            Application.ProcessMessages;						// Acknowledge Cancel or close buttons
        End;

    If ((ReadByte() <> $4B) and (Global_CloseButtonPressed = False))Then // Make sure the ACK 'K' is found
    	Begin
            DoError ('Invalid Response from Amicus18');
            Result := False;
            Exit;
        End;

    If(Global_CloseButtonPressed = True)Then
    	Begin
         	DoError ('Download Cancelled');
            Result := False;
            Exit;
        End;

    Result := ReturnResult;
End;
//----------------------------------------------------------------------------------------
// Write a byte to the serial port
// Returns true if sent OK
Function TDownloadForm.WriteByte(pCharacter: Byte) : Boolean;
Var
	SendBuff : Array [0..1] Of byte;
    Transmitted : dword;
Begin
    SendBuff [0] := pCharacter;
    WriteFile(hCom, SendBuff, 1, Transmitted, nil);   			// send the byte
	Result := Boolean(Transmitted);
End;
//----------------------------------------------------------------------------------------
// Read a byte from the serial port
// Returns the char read
// Returns holding 0 if an error occured
Function TDownloadForm.ReadByte() : Byte;
Var
	RecBuff : Array [0..1] Of byte;
    Received : dword;
Begin
   	ReadFile(hCom, RecBuff, 1, Received, nil);   	    		// receive a byte
    If (Received < 1) Then
     	Result := 0
    else
    	Result := RecBuff[0];
End;
//---------------------------------------------------------------------------------
// Turn progress bar Red
// Display an error message
// Close the com port
Procedure TDownloadForm.DoError(pErrorText: String);
Begin
    WriteButton.Enabled := True;
    CloseButton.Enabled := True;                        // Enable the close button
    Global_DownloadStatus := False;						// Indicate a BAD Write
    DownloadGauge.ForeColor := clRED;
    If(Global_ErrorReported = False) Then 				// Has an error already been issued ?
    	Begin
			Edit2.Caption := pErrorText; 				// No. So issue an error message
    		Application.ProcessMessages();
            Global_ErrorReported := True; 				// Signal that an error has been issued
    	End;
    DelayMs(500);
    CloseCom();
End;
//---------------------------------------------------------------------------------
// Display a message in the window
Procedure TDownloadForm.DisplayMessage(pMessageBuffer: String);
Begin
	Edit2.Caption := pMessageBuffer;
    Application.ProcessMessages();
End;
{----------------------------------------------------------------------------------------}
// Close button pressed
Procedure TDownloadForm.CloseButtonClick(Sender: TObject);
Begin
    Global_CloseButtonPressed := True;
    WriteButton.Enabled := True;
    CloseCom();
	Close();
End;
//---------------------------------------------------------------------------------
// Delay for n Milliseconds
Procedure TDownloadForm.DelayMs(pDelayTime: Integer);
Var
	SentOK: Dword;
Begin
	DelayTimer.Interval := pDelayTime;
    Application.ProcessMessages;
    DelayTimer.Enabled := True;
    While DelayTimer.Enabled Do
    	Begin
            ClearCommError(hCom,SentOK,nil);
        	Application.ProcessMessages;
        End;
End;
{----------------------------------------------------------------------------------------}
// On close Form event
procedure TDownloadForm.FormClose(Sender: TObject;var Action: TCloseAction);
begin
    If (Global_ComPortOpen = True) Then CloseCom();
end;


End.

