unit cMMClient;

interface

uses
   Windows, Forms,Messages,Classes, SysUtils,
   cMMObject, cMMTypes;

type
   TOnPacket = procedure(pSender:TObject;pValue:string) of object;
   TMMClient = class(TMMObject)
   private
      FOnPacket:TOnPacket;
      FOnConnect:TNotifyEvent;
      FOnDisconnect:TNotifyEvent;
      FServerHandle:integer;
      function IsRegistered:boolean;
      function OnServerMessage(var Msg: TMessage):boolean;
   protected
      procedure RegisterClient(pServerHandle:integer);
      procedure DeRegisterClient;
      procedure Connect(pName:string;pSize:Cardinal);
   public
      constructor Create;
      destructor Destroy;override;
      property OnPacket:TOnPacket read FOnPacket write FOnPacket;
      property OnConnect:TNotifyEvent read FOnConnect write FOnConnect;
      property OnDisconnect:TNotifyEvent read FOnDisconnect write FOnDisconnect;
    end;

var
   MMClient:TMMClient;

implementation
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
constructor TMMClient.Create;
begin
   inherited Create('');
   FServerHandle := 0;
   Application.HookMainWindow(OnServerMessage);
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
destructor TMMClient.Destroy;
begin
   Application.UnhookMainWindow(OnServerMessage);
   inherited Destroy;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMClient.OnServerMessage(var Msg: TMessage):boolean;
var
   Value:string;
begin
   result := false;
   if Msg.Msg = FWMServer then
   begin
      result := true;

      // server has data packet (LPARAM is packet ID)...
      if (Msg.WParam = MSG_SERVER_SET_PACKET) and (FServerHandle > 0) then
      begin
         SendMessage(FServerHandle,FWMClient,MSG_CLIENT_GET_PACKET,Msg.LParam);
         Reset;
         Lock;
         Value := ReadString;
         Unlock;
         if Assigned(FOnPacket) then
            FOnPacket(self,Value);
      end

      // server connect acknowledge (LPARAM is server handle)...
      else if Msg.WParam = MSG_SERVER_REG_OK then
      begin
         if Assigned(FOnConnect) and not IsRegistered then
         begin
            FOnConnect(self);
            FServerHandle := Msg.LParam;
         end;
      end

      // server has just started, client is already running (LPARAM is server handle)...
      else if Msg.WParam = MSG_SERVER_CONNECT then
      begin
         if Assigned(FOnConnect) and not IsRegistered then
            FOnConnect(self);
         RegisterClient(Msg.LParam);
      end

      // server has gone offline...
      else if Msg.WParam = MSG_SERVER_DISCONNECT then
      begin
         if Assigned(FOnDisconnect) then
            FOnDisconnect(self);
         FServerHandle := 0;
      end;
   end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMClient.IsRegistered:boolean;
begin
   result := FServerHandle <> 0;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMClient.RegisterClient(pServerHandle:integer);
begin
   FServerHandle := pServerHandle;
   if FServerHandle > 0 then
      PostMessage(FServerHandle,FWMClient,MSG_CLIENT_REGISTER,integer(Application.Handle))
   else
      PostMessage(HWND_BROADCAST,FWMClient,MSG_CLIENT_REGISTER,integer(Application.Handle));
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMClient.DeRegisterClient;
begin
   if FServerHandle > 0 then
      PostMessage(FServerHandle,FWMClient,MSG_CLIENT_DEREGISTER,integer(Application.Handle));
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMClient.Connect(pName:string;pSize:cardinal);
begin
   FName := pName;
   if not Available then
   begin
      FName := pName;
      CreateMap(pSize);
   end;
end;

initialization
   MMClient := TMMClient.Create;
   MMClient.Connect(MM_SERVER_NAME, MM_SERVER_SIZE);
   MMClient.RegisterClient(0);
finalization
   MMClient.DeRegisterClient;
   MMClient.Free;
end.
