unit cMMObject;

interface

uses
   Windows, Classes, SysUtils, cMMTypes, Messages;

type
   TMMObject = class(TObject)
   private
      FFilePointer:cardinal;
      FSize:cardinal;
      FMutexHandle:THandle;
   protected
      FWMServer:cardinal;
      FWMClient:cardinal;
      FName:string;
      FHandle:THandle;
      FMapData:pointer;
      procedure FreeMap;
      procedure IncFP(pSize:cardinal);
      procedure CreateMap(pSize:cardinal);
   public
      constructor Create(pName:string);
      destructor Destroy;override;
      function Available:boolean;
      function Lock:boolean;
      function Unlock:boolean;
      procedure Reset;
      function ReadString:string;
      procedure WriteString(pText:string);
      function ReadInteger: integer;
      procedure WriteInteger(pValue:integer);
      property Name:string read FName;
      property Size:cardinal read FSize;
      property FilePointer:cardinal read FFilePointer;
    end;

implementation
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
constructor TMMObject.Create(pName:string);
begin
   inherited Create;
   FFilePointer := 0;
   FSize := 0;
   FName := pName;
   FMutexHandle := 0;
   FHandle := 0;
   FMapData := nil;
   FWMServer := RegisterWindowMessage(WM_MM_SERVER);
   FWMClient := RegisterWindowMessage(WM_MM_CLIENT);
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
destructor TMMObject.Destroy;
begin
   FreeMap;
   inherited Destroy;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMObject.Available:boolean;
begin
   result := Assigned(FMapData);   
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMObject.Lock:boolean;
begin
   result := WaitForSingleObject(FMutexHandle,500) = WAIT_OBJECT_0;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMObject.Unlock:boolean;
begin
   result := Windows.ReleaseMutex(FMutexHandle);
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.IncFP(pSize:cardinal);
begin
   inc(FFilePointer, pSize);
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.Reset;
begin
   FFilePointer := 0;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.CreateMap(pSize:cardinal);
begin
   if FHandle = 0 then
   begin
      FSize := pSize;
      FHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE,0,FSize,PChar(FName));
      if (FHandle = INVALID_HANDLE_VALUE) or (FHandle = 0) then
      begin
         FMapData := nil;
         FHandle := 0;
         FSize := 0;
      end
      else
      begin
         FMapData := MapViewOfFile(FHandle,FILE_MAP_ALL_ACCESS,0,0,0);
         if Assigned(FMapData) then
            FHandle := CreateMutex(nil,false,PChar(FName + 'Mutex'))
         else
         begin
            CloseHandle(FHandle);
            FHandle := 0;
            FSize := 0;
         end;
      end;
   end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.FreeMap;
begin
   if FHandle <> 0 then
   begin
      UnmapViewOfFile(FMapData);
      CloseHandle(FHandle);
      FSize := 0;
      FFilePointer := 0;
   end;
   if FMutexHandle <> 0 then
   begin
      CloseHandle(FMutexHandle);
      FMutexHandle := 0;
   end;   
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMObject.ReadString:string;
var
   ptr:pointer;
begin
   result := '';
   if Assigned(FMapData) then
   begin
      ptr := pointer(cardinal(FMapData) + FilePointer);
      result := string(PChar(ptr));
      IncFP(Length(result) + 1);
   end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.WriteString(pText:string);
var
   ptr:pointer;
begin
   if Assigned(FMapData) then
   begin
      ptr := pointer(cardinal(FMapData) + FilePointer);
      StrLCopy(ptr,PChar(pText),Size);
      IncFP(Length(pText) + 1);
   end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMObject.ReadInteger: integer;
var
  ptr:pointer;
begin
   if Assigned(FMapData) then
   begin
      ptr := pointer(cardinal(FMapData) + FilePointer);
      CopyMemory(@Result, ptr, sizeof(integer));
      IncFP(sizeof(integer));
   end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMObject.WriteInteger(pValue: integer);
var
  ptr:pointer;
begin
   if Assigned(FMapData) then
   begin
      ptr := pointer(cardinal(FMapData) + FilePointer);
      CopyMemory(ptr, @pValue, sizeof(integer));
      IncFP(sizeof(integer));
   end;
end;


end.
