unit cMMServer;

interface

uses
   Forms,Windows, Messages,Classes, SysUtils,
   cMMObject, cMMTypes;

type
   TMMServer = class(TMMObject)
   private
      FClients:TList;
      FBuffer:TStringList;
      FBufferIndex:integer;
      procedure InitBuffer;
      function GetBufferStr(pIndex:integer):string;
      function HaveClients:boolean;
      function OnClientMessage(var Msg: TMessage):boolean;
   public
      constructor Create(pName:string;pSize:cardinal);
      destructor Destroy;override;
      procedure WritePacket(pValue:string);
    end;
var
   MMServer:TMMServer;

implementation

{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
constructor TMMServer.Create(pName:string;pSize:cardinal);
begin
   inherited Create(pName);
   CreateMap(pSize);
   FClients := TList.Create;
   FBuffer := TStringList.Create;
   InitBuffer;
   Application.HookMainWindow(OnClientMessage);
   PostMessage(HWND_BROADCAST,FWMServer,MSG_SERVER_CONNECT,integer(Application.Handle));
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
destructor TMMServer.Destroy;
begin
   Application.UnhookMainWindow(OnClientMessage);
   PostMessage(HWND_BROADCAST,FWMServer,MSG_SERVER_DISCONNECT,0);
   FBuffer.Free;
   FClients.Free;
   inherited Destroy;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMServer.InitBuffer;
var
   Index:integer;
begin
   FBuffer.Clear;
   for Index := 0 to 511 do
      FBuffer.Add('');
   FBufferIndex := 0;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMServer.GetBufferStr(pIndex:integer):string;
begin
   result := '';
   if pIndex < FBuffer.Count then
      result := FBuffer.Strings[pIndex];
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMServer.OnClientMessage(var Msg: TMessage):boolean;
var
   ClientHandle:integer;
begin
   result := false;
   if Msg.Msg = FWMClient then

      // client wants to connect to server...
      if Msg.WParam = MSG_CLIENT_REGISTER then
      begin
         result := true;
         ClientHandle := Msg.LParam;
         FClients.Add(pointer(ClientHandle));
         PostMessage(ClientHandle,FWMServer,MSG_SERVER_REG_OK,integer(Application.Handle));
      end

      // client wants to disconnect from server...
      else if Msg.WParam = MSG_CLIENT_DEREGISTER then
      begin
         result := true;
         ClientHandle := Msg.LParam;
         FClients.Remove(pointer(ClientHandle));
      end

      // client wants a packet...
      else if Msg.WParam = MSG_CLIENT_GET_PACKET then
      begin
         result := true;
         Reset;
         Lock;
         WriteString(GetBufferStr(Msg.LParam));
         Unlock;
      end;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
function TMMServer.HaveClients:boolean;
begin
   result := FClients.Count > 0;
end;
{
****************************************************************************
* Name    :                                                                *
* Purpose :                                                                *
****************************************************************************
}
procedure TMMServer.WritePacket(pValue:string);
var
   Index, ClientHandle:integer;
begin
   FBuffer.Strings[FBufferIndex] := pValue;
   if HaveClients then
      for Index := 0 to FClients.Count - 1 do
      begin
         ClientHandle := integer(FClients.Items[index]);
         PostMessage(ClientHandle,FWMServer,MSG_SERVER_SET_PACKET,FBufferIndex);
      end;
   inc(FBufferIndex);
   if FBufferIndex = FBuffer.Count then
      FBufferIndex := 0;
(*
   if HaveClients then
      for Index := 0 to FClients.Count - 1 do
      begin
         ClientHandle := integer(FClients.Items[index]);
         Reset;
         Lock;
         WriteString(pValue);
         Unlock;
         SendMessage(ClientHandle,FWMServer,MSG_SERVER_PACKET,0);
      end;
*)      
end;

// create and destroy server object...
initialization
   MMServer := TMMServer.Create(MM_SERVER_NAME,MM_SERVER_SIZE);
finalization
   MMServer.Free;
end.
