;
; Amicus18 - 18F25K20 Bootloader
; Based upon the tiny bootloader firmware
;
 list  P = 18F25K20, F = INHX32, W = 2, X = ON, R = DEC, MM = ON, N = 0, C = 255, T = ON

;---------------------------------------------------------------------------
PIR1             EQU  H'0F9E'
EECON1           EQU  H'0FA6'
EECON2           EQU  H'0FA7'
EEDATA           EQU  H'0FA8'
EEADR            EQU  H'0FA9'
RCSTA            EQU  H'0FAB'
TXSTA            EQU  H'0FAC'
TXREG            EQU  H'0FAD'
RCREG            EQU  H'0FAE'
SPBRG            EQU  H'0FAF'
STATUS           EQU  H'0FD8'
FSR0L            EQU  H'0FE9'
FSR0H            EQU  H'0FEA'
PREINC0          EQU  H'0FEC'
POSTINC0         EQU  H'0FEE'
INDF0            EQU  H'0FEF'
TABLAT           EQU  H'0FF5'
TBLPTR           EQU  H'0FF6'
TBLPTRL          EQU  H'0FF6'
TBLPTRH          EQU  H'0FF7'
TBLPTRU          EQU  H'0FF8'

RD    = 0
WR    = 1
WREN  = 2
EEPGD = 7
TXIF  = 4
RCIF  = 5
;---------------------------------------------------------------------------
 variable IdTypePIC = 0X4A
 #define MaxFlash 0x8000

 variable XtalFreq = 64000000           ; Crystal frequency
 variable BaudRate = 115200             ; Standard BaudRate rate of 115200


 #define StartAddress MaxFlash - 200   ; 100 words

;---------------------------------------------------------------------------
 variable RoundResult = 0                ; Rounding macro.  RoundResult = Round(aa/bb)
Round Macro aa,bb
 Local rr = aa/bb
 Local d1 = aa - rr * bb
 Local d2 = (rr + 1) * bb - aa
 variable RoundResult = rr
    if (d1 >= d2)
 RoundResult++
    endif
    Endm
;---------------------------------------------------------------------------
 variable AbsResult = 0
Abs Macro nr
    if (nr >= 0)
 AbsResult = nr
    else
 AbsResult = -nr
    endif
    Endm
;---------------------------------------------------------------------------
    Round XtalFreq,(16 * BaudRate)
 variable SPBRG_Value = RoundResult - 1

    Round XtalFreq,(16 * (SPBRG_Value + 1))
 variable BaudRate_real = RoundResult
    Abs(BaudRate_real - BaudRate)
 variable errpercent = AbsResult * 100 / BaudRate
    if (errpercent > 4)
      Error Big error in BaudRaterate: #v(errpercent)%
    endif
    if (errpercent >= 2)
      Messg BaudRaterate not exact: #v(errpercent)%
    endif
;---------------------------------------------------------------------------
; Variables
 Cblock 0
    CRC
    AmountOfWrites
    TimeoutCountHH
    TimeoutCountH
    TimeoutCount
    CounterH
    CounterL
    Flags
 Endc

 Cblock 10
    Buffer:64
    Dummy4Crc
 Endc
;---------------------------------------------------------------------------
TransmitByte Macro pCharacter
    Movlw pCharacter
    Movwf TXREG
    Endm

; Program
;---------------------------------------------------------------------------
    Org     0x0000
    Goto    Start_Of_Bootloader

;---------------------------------------------------------------------------
; Start
    Org     StartAddress
    Nop
    Nop
    Nop
    Nop

;---------------------------------------------------------------------------

    Org StartAddress + 8
Start_Of_Bootloader

; Init USART
    Movlw     0x24
    Movwf     TXSTA
    Movlw     SPBRG_Value
    Movwf     SPBRG
    Movlw     0x90
    Movwf     RCSTA

; Wait for computer
    Rcall     ReceiveByte            ; Receive a byte from the PC
    Sublw     0xC1                   ; Expect 0xC1
    Bnz       StartAddress
    TransmitByte IdTypePIC           ; Transmit the PICmicro type

MainLoop1
    TransmitByte 'K'                 ; Send an acknowledge character 'K'
MainLoop2
    Clrf      CRC                    ; Clear the CRC checksum before we start
    Rcall     ReceiveByte            ; Receive the Upper byte address for TBLPTR (TBLPTRU)
    Movwf     TBLPTRU
    Movwf     Flags                  ; This is also the byte used by Flags
    Rcall     ReceiveByte            ; Receive the High byte address for TBLPTR (TBLPTRH)
    Movwf     TBLPTRH
    Movwf     EEADR                  ; Also place Data address byte in Data memory access register
    Rcall     ReceiveByte            ; Receive the Low byte address for TBLPTR (TBLPTRL)
    Movwf     TBLPTRL
    Movwf     EEDATA                 ; Also place Low byte in Data memory access register

    Rcall     ReceiveByte            ; Receive the amount of writes to perform
    Movwf     AmountOfWrites
    Incf      AmountOfWrites,f       ; Increment the amount of writes by 1
    Lfsr      0,(Buffer - 1)         ; Point FSR0 to the start of the Buffer

RcvCct                               ; Read 64 + 1 bytes
    Movwf     TABLAT                 ; Prepare for cfg; => store byte before CRC
    Rcall     ReceiveByte            ; Receive the byte to write to the PICmicro memory
    Movwf     PREINC0                ; Place the byte into Buffer's RAM
    Decfsz    AmountOfWrites,f
    Bra       RcvCct                 ; Loop for 65 bytes in Buffer. Last byte is CRC

    Tstfsz    CRC                    ; Check CRC (last byte received)
    Bra       TransmitError
    Btfss     Flags,6                ; Flags.6 set for Data memory access
    Bra       NoEeprom
    Movlw     4                      ; Setup for eeprom in register EECON1
    Rcall     SetupEeprom
    Bra       WaitForWrite
NoEeprom
    Btfss     Flags,7                ; Flags.7 set for Config memory access
    Bra       NoConfig
    Tblwt*                           ; Write TABLAT (byte before CRC) to TBLPTR
    Movlw     0xC4                   ; Setup for Config in register EECON1
    Rcall     SetupEeprom
    Bra       WaitForWrite
NoConfig

;--------------------------------------------------------------------------------

WriteBufferToMemory
    Movlw     0x94                   ; Setup for Erase in register EECON1
    Rcall     SetupEeprom
    Tblrd*-                          ; Point to address - 1

    Movlw     8                      ; 8 groups
    Movwf     CounterH
    Lfsr      0,Buffer               ; Point FSR0 to start of Buffer
WriteInnerLoop
    Movlw     8                      ; 8 bytes = 4 instructions
    Movwf     CounterL
WriteByte
    Movf      POSTINC0,w             ; Get a byte from the Buffer
    Movwf     TABLAT
    Tblwt+*                          ; Write it to memory
    Decfsz    CounterL               ; 8 bytes must be written in one operation
    Bra       WriteByte

    Movlw     0x84                   ; Setup for writes in register EECON1
    Rcall     SetupEeprom
    Decfsz    CounterH,f
    Bra       WriteInnerLoop
WaitForWrite
    Btfsc     EECON1,WR              ; For eeprom writes (wait to finish write)
    Bra       $ - 2                  ; No need: round trip time with PC bigger than 4ms

    Bcf       EECON1,WREN            ; Disable writes
    Bra       MainLoop1

TransmitError
    TransmitByte 'N'                 ; Transmit N
    Bra      MainLoop2
;-------------------------------------------------------------------------------
SetupEeprom
    Movwf     EECON1
    Movlw     0x55
    Movwf     EECON2
    Movlw     0xAA
    Movwf     EECON2
    Bsf       EECON1,WR              ; Set for perform Write
    Bra       $ + 2
    Return

;-------------------------------------------------------------------------------
; Receive a byte from the serial port
; Byte is returned in WREG
; CRC checksum is updated on every byte received
ReceiveByte
    Movlw     (XtalFreq / 1000000);
    Movwf     TimeoutCountHH
TimeoutLoop1
    Clrf      TimeoutCountH
TimeoutLoop2
    Clrf      TimeoutCount
TimeoutLoop3
    Btfss     PIR1,RCIF              ; Test RX
    Bra       NotReceived
    Movf      RCREG,w                ; Return in WREG
    Addwf     CRC,f                  ; Calculate the CRC checksum
    Return
NotReceived
    Decfsz    TimeoutCount,f
    Bra       TimeoutLoop3
    Decfsz    TimeoutCountH,f
    Bra       TimeoutLoop2
    Decfsz    TimeoutCountHH,f
    Bra       TimeoutLoop1
    Clrf      TXSTA
    Bra       StartAddress

;-------------------------------------------------------------------------------

 CONFIG DEBUG = OFF      ; Background debugger disabled; RB6 and RB7 configured as general purpose I/O pins
 CONFIG XINST = OFF      ; Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
 CONFIG STVREN = OFF     ; Reset on stack overflow/underflow disabled
 CONFIG WDTEN = OFF      ; WDT disabled (control is placed on SWDTEN bit)
 CONFIG FCMEN = OFF      ; Fail-Safe Clock Monitor disabled
 CONFIG FOSC = HSPLL     ; HS oscillator, PLL enabled and under software control
 CONFIG IESO = OFF       ; Two-Speed Start-up disabled
 CONFIG WDTPS = 128      ; 1:128 prescaller for watchdog
 CONFIG BOREN = OFF      ; Brown-out Reset disabled in hardware and software
 CONFIG BORV = 18        ; VBOR set to 1.8 V nominal
 CONFIG MCLRE = ON       ; MCLR pin enabled, RE3 input pin disabled
 CONFIG HFOFST = OFF     ; The system clock is held off until the HF-INTOSC is stable.
 CONFIG LPT1OSC = ON     ; Timer1 operates in standard power mode
 CONFIG PBADEN = OFF     ; PORTB<4:0> pins are configured as digital I/O on Reset
 CONFIG CCP2MX = PORTC   ; CCP2 input/output is multiplexed with RC1
 CONFIG LVP = OFF        ; Single-Supply ICSP disabled
 CONFIG CP0 = OFF        ; Block 0 (000800-001FFFh) not code-protected
 CONFIG CP1 = OFF        ; Block 1 (002000-003FFFh) not code-protected
 CONFIG CPB = OFF        ; Boot block (000000-0007FFh) not code-protected
 CONFIG CPD = OFF        ; Data eeprom not code-protected
 CONFIG WRT0 = OFF       ; Block 0 (000800-001FFFh) not write-protected
 CONFIG WRT1 = OFF       ; Block 1 (002000-003FFFh) not write-protected
 CONFIG WRTB = OFF       ; Boot block (000000-0007FFh) not write-protected
 CONFIG WRTC = OFF       ; Configuration registers (300000-3000FFh) not write-protected
 CONFIG WRTD = OFF       ; Data eeprom not write-protected
 CONFIG EBTR0 = OFF      ; Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
 CONFIG EBTR1 = OFF      ; Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
 CONFIG EBTRB = OFF      ; Boot block (000000-0007FFh) not protected from table reads executed in other blocks
;-------------------------------------------------------------------------------
    End

